/*
 * Copyright 2011 Endea
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea

import entity.EntityIndex
import entity.StringIndex

object Document {

  val parents = EntityIndex[Document, Document]("sections")
  val tags = StringIndex[Document]("tags")
}

class Document(
  var name: String) extends Entity {

  private var _text: String = _

  def text = _text

  def text_=(text: String) {
    _text = text
  }

  @PType(classOf[Document])
  @Indexed
  var sections: IndexedSeq[Document] = IndexedSeq.empty

  @PType(classOf[String])
  @Indexed
  var tags: List[String] = Nil

  @PType(classOf[Document])
  @Indexed
  private var documents: List[Document] = Nil

  @PType(classOf[Image])
  @Indexed
  private var images: List[Image] = Nil

  def parents() = Document.parents.get(this)
}