/*
 * Copyright (c) 2012, Endea.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.html

import endea._
import endea.http._
import endea.internal.entity._
import entity.EntityIndex
import entity.ClassIndex
import scala.collection.immutable.Stream.Empty

object EntityTypeAction extends HtmlAction[Type[Entity]] {

  val css = None
  val js = None

  def write(event: Event[Type[Entity]]) {

    val _clazz = event.entity._class

    if (_clazz == classOf[Entity]) {
      writeEntity(event)

    } else
      query(event, _clazz)
  }

  private def query(event: Event[_], clazz: Class[_]) {

    if (event.parameters.size > 0)
      attributeSearch(event, clazz)
    else
      classSearch(event, clazz)
  }

  private def attributeSearch(event: Event[_], clazz: Class[_]) {

    val (pname, pvalue) = event.parameters.head

    val optAttribute = MetaEntity.getByClass(clazz).findAttribute(pname)
    if (optAttribute.isDefined) {
      val attribute = optAttribute.get
      if (attribute.index.isDefined) {
        val value = if (pvalue.size > 0) pvalue(0) else null
        IndexAdapter.write(event, attribute.index.get, pname, value)
      } else
        event.write("<p>No indexed attribute: " + pname + "</p>")
    } else
      event.write("<p>Unknow attribute: " + pname + "</p>")
  }

  private def classSearch(event: Event[_], clazz: Class[_]) {

    event.write("""<b>indexes: &nbsp; &nbsp; </b> """)

    for (attribute <- MetaEntity.getByClass(clazz).attributes) {
      val option = attribute.index
      if (option.isDefined) {
        event.write("""<a href="/""")
        event.write(clazz.getSimpleName.toLowerCase)
        event.write("?")
        event.write(attribute.name)
        event.write("""">""")
        event.write(attribute.name)
        event.write("</a>  &nbsp; ")
      }
    }

    var rows = 10

    event.write("<hr/>")

    event.write("<ul>")
    ClassIndex.get(clazz.asInstanceOf[Class[Entity]], _.take(rows).foreach(
      entity => {
        event.write("<li>")
        Link.write(event, entity)
        event.write("</li>")
      }))
    event.write("</ul>")
  }

  private def writeEntity(event: endea.http.Event[endea.http.Type[endea.Entity]]): Unit = {

    event.write("<ul>")
    for (clazz <- MetaEntity.all) {
      event.write("""<li><a href="/""")
      event.write(clazz.getSimpleName().toLowerCase)
      event.write("""">""")
      event.write(clazz.getSimpleName())
      event.write("""</a></li>
""")
    }
    event.write("</ul>")
  }
}