/*
 * Copyright 2012 Endea
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.html

import endea._
import endea.http._

abstract class HtmlAction[E <: Entity](
  implicit manifest: Manifest[E]) extends HttpAction[E] {

  def css: Option[ClasspathResource]

  def js: Option[ClasspathResource]

  def controls(event: Event[E]): Seq[Control] = Nil

  def write(event: Event[E])

  def doGet(event: Event[E]) {

    event.write("""<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8" />
  <title>""")
    event.write(HtmlConfig.title)
    event.write("""</title>
  <link rel="shorcut icon" type="image/x-icon" href="""")
    write(event, HtmlConfig.favicon, """"/>""")
    writeCss(event, HtmlConfig.css)
    writeCss(event, css)
    // <link rel="apple-touch-icon" href="/apple-touch-icon.png"/>
    event.write("""  </head>
<body>
  <div class="e-nav">
""")
    nav(event)
    event.write("""  </div>
  <div class="e-main""")
    if (HtmlConfig.adSense.isDefined)
      event.write(" e-main-adsense")
    event.write("""">
""")
    main(event)
    event.write("""  </div>
""")
    event.write("""<script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.7.1/jquery.min.js"></script>
""")
    event.write("""<script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.16/jquery-ui.min.js"></script>
""")
    writeJs(event, js)
    adSense(event)
    event.write("""</body>
</html>""")
  }

  private def nav(event: Event[E]) {

    for (control <- HtmlConfig.controls)
      if (control.contribute(event))
        control.write(event)

    for (control <- controls(event))
      if (control.contribute(event))
        control.write(event)
  }

  def main(event: Event[E]) {

    write(event)
  }

  private def adSense(event: Event[E]) {

    if (HtmlConfig.adSense.isDefined) {
      val adSense = HtmlConfig.adSense.get
      event.write("""<div class="e-adsense">
<script type="text/javascript"><!--
google_ad_client = """")
      event.write(adSense.client)
      event.write("""";
/* Autoexamen */
google_ad_slot = """")
      event.write(adSense.slot)
      event.write("""";
google_ad_width = 120;
google_ad_height = 600;
//-->
</script>
<script type="text/javascript"
src="http://pagead2.googlesyndication.com/pagead/show_ads.js">
</script>
</div>
""")
    }
  }

  private def writeCss(event: Event[E], option: Option[ClasspathResource]) {

    if (option.isDefined) {
      event.write("""<link rel="stylesheet" type="text/css" href="""")
      option.get.write(event)
      event.write("""" />
""")

    }
  }

  private def writeJs(event: Event[E], option: Option[ClasspathResource]) {

    if (option.isDefined) {
      event.write("""<script type="text/javascript" src="""")
      option.get.write(event)
      event.write(""""></script>
""")
    }
  }

  private def write(event: Event[E], resource: Option[ClasspathResource], string: String) {
    ClasspathResource.write(event, resource)
    event.write(string)
  }
}