/*
 * Copyright (c) 2011, Endea.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.internal.html

import endea._
import endea.html._
import endea.http.Event
import endea.http.ClasspathResource

object DocumentAction extends HtmlAction[Document] {

  val css = ClasspathResource(DocumentAction, "document.css")
  val js = None

  override def write(event: Event[Document]) {

    val document = event.entity

    

    val parents = document.parents()
    if (!parents.isEmpty) {
      event.write("""<div class="nav">""")
      for (parent <- parents) {
        event.write("""<div class="parent">""")
        Link.write(event, parent)
        event.write("</div>")
        event.write("<ul>")
        for (section <- parent.sections) {
          if (section == document) {
            event.write("""<li class="sel">""")
            event.write(document.name)
          } else {
            event.write("<li>")
            Link.write(event, section)
          }
          event.write("</li>")
        }
        event.write("</ul>")
      }
      event.write("</div>")
    }

    event.write("""<div id="text">""")
    
    event.write("""<h2 id="name">""")
    event.write(document.name)
    event.write("</h2>")

    if (!document.sections.isEmpty) {
      event.write("<ul>")
      for (section <- document.sections) {
        event.write("<li>")
        Link.write(event, section)
        event.write("</li>")
      }
      event.write("</ul>")
    }

    if (document.text != null)
      event.write(document.text)
    event.write("<div>")

  }
}