/*
 * Copyright (c) 2012, Endea.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.internal.http

import endea._
import endea.html._
import endea.http._
import endea.internal._
import endea.internal.entity._
import endea.internal.html._
import endea.internal.http._
import scala.collection.mutable.HashMap
import scala.collection.mutable.ArrayBuffer

object ActionMap {

  private val actions = new HashMap[String, HttpAction[_]]
  private val typeActions = new HashMap[String, HttpAction[_]]

  private val typeName = classOf[Type[_]].getName()
  private val split = typeName.length + 1

  for (action <- HttpConfig.actions) {

    var name = action.className
    var typeAction = false
    if (name.startsWith(typeName)) {
      name = name.substring(split, name.length - 1)
      typeAction = true
    }

    val map = if (typeAction) typeActions else actions
    map += name + "/" + action.name -> action
  }

  def getAction[E <: Entity](clazz: Class[_], action: String, typeAction: Boolean): HttpAction[E] = {

    val map = if (typeAction) typeActions else actions

    val option = map.get(clazz.getName() + "/" + action)
    if (option.isDefined)
      option.get.asInstanceOf[HttpAction[E]]
    else
      getAction(clazz.getSuperclass().asInstanceOf[Class[E]], action, typeAction)
  }
}