/*
 * Copyright 2011 Endea
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package endea.internal.secure

import endea._
import endea.http._
import endea.html._
import endea.internal.http._

object LoginAction extends HtmlAction[Type[User]] {

  override val name = "log-in"

  val css = ClasspathResource(LoginAction, "login.css")
  val js = None

  private val url = Controller.encode(LoginAction)
  private val nameId = "name"
  private val passwordId = "password"

  override def doPost(event: Event[Type[User]]) {

    var user: User = null

    val optName = event.parameter(nameId)
    if (optName.isDefined) {
      event.attribute(nameId, optName.get)
      val optPassword = event.parameter(passwordId)
      if (optPassword.isDefined) {
        val optUser = User.logIn(event, optName.get, optPassword.get)
        if (optUser.isDefined)
          user = optUser.get
      }
    }

    if (user == null)
      doGet(event)
    else
      event.httpResponse.sendRedirect(User.user_uri)
  }

  def write(event: Event[Type[User]]) {

    event.write("""<div class="login">
<table class="panel">
<form action="""")
    event.write(url)
    event.write("""" method="post">
  <div style="display:none">
  <input type="hidden" name="csrfmiddlewaretoken" value=""></div>
  <tr><td></td></tr>
  <tr><td class="label"><label for="name">Name</label></td></tr>
  <tr><td><input type="text" id="name" name="""")
    event.write(nameId)
    event.attribute[String](nameId) match {
      case Some(id) =>
        event.write("""" value="""")
        event.write(id)
      case _ =>
    }
    event.write("""" autofocus="autofocus"/></tr></td>
  <tr><td class="label"><label for="password">Password</label></td></tr>
  <tr><td><input type="password" id="password" name="""")
    event.write(passwordId)
    event.write(""""/></tr></td>
  <tr><td><input type="submit" value="Log in" name="submit"/></tr></td>
</form>
</table>
 <!--p class="reset"><a href="/account/password/reset/" tabindex="-1">Reset password</a></p-->        
<!--br/><p>Need an account? <a href="">Sign up</a> now!</p-->
</div>""")
  }
}